#include "StdAfx.h"
#include "Bar.h"
#include "IndicatorBarSource.h"
#include "ResponseListener.h"
#include "LiveTicksUpdater.h"
#include "FXConnectHistoryLoader.h"

FXConnectHistoryLoader::FXConnectHistoryLoader(IO2GSession *session)
{
    mSession = session;
    mSession->addRef();
}

FXConnectHistoryLoader::~FXConnectHistoryLoader(void)
{
    mSession->release();
}

void FXConnectHistoryLoader::getHistoricalData(CIndicatorBarSource *source,
                                               DATE dateFrom,
                                               DATE dateTo)
{
    O2G2Ptr<ResponseListener> responseListener = new ResponseListener(mSession);
    O2G2Ptr<IO2GRequestFactory> requestFactory = mSession->getRequestFactory();
    if (requestFactory)
    {
        O2G2Ptr<IO2GTimeframeCollection> timeframeCollection = requestFactory->getTimeFrameCollection();
        O2G2Ptr<IO2GTimeframe> timeframe = timeframeCollection->get(source->getTimeframe());
        if (timeframe)
        {
            bool bComplete = false;
            DATE dFrom = dateFrom, dTo = 0;
            std::list<std::vector<Bar*> > barChunks;

            mSession->subscribeResponse(responseListener);

            while (!bComplete)
            {
                O2G2Ptr<IO2GRequest> request = requestFactory->createMarketDataSnapshotRequestInstrument(source->getInstrument(), timeframe, 300);
                if (request)
                {
                    requestFactory->fillMarketDataSnapshotRequestTime(request, dFrom, dTo);

                    responseListener->setRequest(request->getRequestID());
                    mSession->sendRequest(request);

                    WaitForSingleObject(responseListener->getRequestEvent(), 10000);
                    if (!responseListener->hasError())
                    {
                        O2G2Ptr<IO2GResponse> response = responseListener->getLastResponse();
                        O2G2Ptr<IO2GResponseReaderFactory> readerFactory = mSession->getResponseReaderFactory();
                        if (readerFactory)
                        {
                            O2G2Ptr<IO2GMarketDataSnapshotResponseReader> reader = \
                                readerFactory->createMarketDataSnapshotReader(response);

                            std::vector<Bar*> barChunk;
                            for (int i = 0; i < reader->size(); ++i)
                            {
                                Bar *bar = new Bar();

                                bar->date = reader->getDate(i);
                                bar->volume = reader->getVolume(i);

                                if (source->isBid())
                                {
                                    bar->close = reader->getBidClose(i);
                                    bar->open = reader->getBidOpen(i);
                                    bar->high = reader->getBidHigh(i);
                                    bar->low = reader->getBidLow(i);
                                }
                                else
                                {
                                    bar->close = reader->getAskClose(i);
                                    bar->open = reader->getAskOpen(i);
                                    bar->high = reader->getAskHigh(i);
                                    bar->low = reader->getAskLow(i);
                                }

                                barChunk.push_back(bar);
                            }

                            barChunks.push_back(barChunk);

                            // Check if all data received, update date borders for the next iteration.
                            if (reader->getDate(0) - dFrom < 0.000001)
                                bComplete = true;
                            else
                                dTo = reader->getDate(0);
                        }
                        else
                        {
                            mError = "FXConnectHistoryLoader::getHistoricalData(): invalid session.";
                            bComplete = true;
                        }
                    }
                    else
                    {
                        mError = "FXConnectHistoryLoader::getHistoricalData(): request failed: ";
                        mError += responseListener->getError();
                        bComplete = true;
                    }
                }
                else
                {
                    mError = "FXConnectHistoryLoader::getHistoricalData(): failed to create request.";
                    bComplete = true;
                }
            }
            mSession->unsubscribeResponse(responseListener);

            // organize chunks to solid bar source.
            for (std::list<std::vector<Bar*> >::reverse_iterator i = barChunks.rbegin(); i != barChunks.rend(); ++i)
            {
                for (std::vector<Bar*>::iterator j = i->begin(); j != i->end(); ++j)
                    source->addBar(*j);
            }
        }
        else
            mError = "FXConnectHistoryLoader::getHistoricalData(): invalid timeframe.";
    }
    else
        mError = "FXConnectHistoryLoader::getHistoricalData(): invalid instrument.";
}

const char *FXConnectHistoryLoader::getError()
{
    return mError.c_str();
}