#include "stdafx.h"
#include "Bar.h"
#include "IndicatorBarSource.h"
#include "HistoryLoader.h"
#include "LiveTicksUpdater.h"
#include "FXConnectHost.h"

FXConnectHost::FXConnectHost(IO2GSession *session, const std::string &csvSource) : BaseHostImpl()
{
    mSession = session;
    if (mSession)
        mSession->addRef();
    mCSVSource = csvSource;

    /*
    The offset in hours of the beginning of the trading day against the calendar Midnight.
    For example, FXCM's trading day starts at 17:00 EST of the previous day, so the offset if -7 hours.
    */
    mTradingDayOffset = -7;

    /*
    The offset in days of the beginning of the trading week against the Monday.
    For example, FXCM's trading week starts Saturday 17:00 EST (i.e. Sunday trading day),
    so the offset is -1 day. This parameter is used only in case the unit is Week.
    */
    mTradingWeekOffset = -1;

    mLiveTicksUpdater = new LiveTicksUpdater(mSession, mTradingDayOffset, mTradingWeekOffset);
    mLoader = NULL;
    mLoaderFinishedEvent = CreateEvent(0, FALSE, TRUE, 0);
}

FXConnectHost::~FXConnectHost(void)
{
    if (mSession)
        mSession->release();

    if (mLoader)
    {
        if (!mLoader->isStopRequested())
            mLoader->requestStop();
        mLoader->join();

        delete mLoader;
    }
    CloseHandle(mLoaderFinishedEvent);
}

IIndicatorSource *FXConnectHost::getHistory(IInstance *instance, int cookie, const char *instrument, const char *period, DATE from, DATE to, bool bid, const char **error)
{
    std::string err;
    CIndicatorBarSource *source = NULL;

    O2G2Ptr<IO2GOfferRow> offer = getOffer(instrument);
    if (offer)
    {
        ResetEvent(mLoaderFinishedEvent);
        source = new CIndicatorBarSource(period, instrument, offer->getPointSize(), offer->getDigits(), bid);
        mLoader = new HistoryLoader(instance, cookie, source, from, to, mSession, mCSVSource, mLoaderFinishedEvent);
        if (to == 0.0)
            mLiveTicksUpdater->subscribeOfferUpdates((IIndicatorInstance*)instance, source);
        mLoader->start();
        err = mLoader->getError();
    }
    else
        err = "FXConnectHost::getHistory(): invalid instrument.";

    *error = err.c_str();
    return source;
}

HANDLE FXConnectHost::getLoaderFinishedEvent()
{
    return mLoaderFinishedEvent;
}

IO2GOfferRow *FXConnectHost::getOffer(const char *instrument)
{
    if (!mSession || !instrument)
        return NULL;

    O2G2Ptr<IO2GLoginRules> loginRules = mSession->getLoginRules();
    if (loginRules)
    {
        O2G2Ptr<IO2GResponse> response = loginRules->getTableRefreshResponse(Offers);
        if (response)
        {
            O2G2Ptr<IO2GResponseReaderFactory> readerFactory = mSession->getResponseReaderFactory();
            if (readerFactory)
            {
                O2G2Ptr<IO2GOffersTableResponseReader> reader = readerFactory->createOffersTableReader(response);

                for (int i = 0; i < reader->size(); ++i)
                {
                    O2G2Ptr<IO2GOfferRow> offer = reader->getRow(i);
                    if (offer)
                        if (strcmp(instrument, offer->getInstrument()) == 0)
                            if (strcmp(offer->getSubscriptionStatus(), "T") == 0 || 
                                strcmp(offer->getSubscriptionStatus(), "V") == 0)
                                return offer.Detach();
                }
            }
        }
    }
    return NULL;
}

int FXConnectHost::getTradingDayOffset()
{
    return mTradingDayOffset;
}

int FXConnectHost::getTradingWeekOffset()
{
    return mTradingWeekOffset;
}
