// ForexConnectHost.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "Bar.h"
#include "IndicatorBarSource.h"
#include "HistoryLoader.h"
#include "FXConnectHost.h"
#include "FXConnectHistoryLoader.h"
#include "SessionStatusListener.h"
#include "LoginParams.h"
#include "SampleParams.h"
#include <iomanip>

namespace {

bool Login(IO2GSession *session, SessionStatusListener *statusListener, LoginParams *loginParams);
void Logout(IO2GSession *session, SessionStatusListener *listener);
IO2GOfferRow *GetOffer(IO2GSession *session, const char *instrument);
const char* FormatDate(double dt);
void PrintResults(IIndicatorInstance *instance);
bool CheckObligatoryParams(LoginParams*, SampleParams*);
void PrintHelp();

int ForexConnectHost(int argc, char* argv[])
{
    if (argc == 1)
    {
        PrintHelp();
        return 0;
    }

    LoginParams *loginParams = new LoginParams(argc, argv);
    SampleParams *sampleParams = new SampleParams(argc, argv);

    if (!CheckObligatoryParams(loginParams, sampleParams))
        return -1;

    IO2GSession *session = CO2GTransport::createSession();
    SessionStatusListener *sessionStatusListener = new SessionStatusListener(session, true, loginParams->getSessionID(), loginParams->getPin());
    session->subscribeSessionStatus(sessionStatusListener);

    if (Login(session, sessionStatusListener, loginParams))
    {
        IndicoreManager* manager = IndicoreManager::getInstance("enu");

        FXConnectHost *host = new FXConnectHost(session, sampleParams->getCSVSource());
        manager->setHost(host);

        manager->loadIndicators(sampleParams->getIndicatorsPath());
        IIndicators* allIndicators = manager->getIndicators();

        FXConnectHistoryLoader *fxcHistoryLoader = new FXConnectHistoryLoader(session);
        O2G2Ptr<IO2GOfferRow> offer = GetOffer(session, sampleParams->getInstrument());
        if (offer)
        {
            O2G2Ptr<IO2GRequestFactory> requestFactory = session->getRequestFactory();
            O2G2Ptr<IO2GTimeframeCollection> timeframeCollection = requestFactory->getTimeFrameCollection();
            O2G2Ptr<IO2GTimeframe> timeframe = timeframeCollection->get(sampleParams->getTimeframe());
            if (timeframe)
            {
                CIndicatorBarSource* barSource = new CIndicatorBarSource(timeframe->getID(), offer->getInstrument(), offer->getPointSize(), offer->getDigits(), sampleParams->isBid());

                std::cout << "Loading historical data...";
                fxcHistoryLoader->getHistoricalData(barSource, sampleParams->getDateFrom(), sampleParams->getDateTo()); 
                std::cout << " done!" << std::endl;
                delete fxcHistoryLoader;
                
                IIndicator* indicator = allIndicators->findIndicator("PIVOT");
                if (indicator)
                {
                    IIndicatorParameters* indParams = indicator->getParameters();
                    IIndicatorParameter* modeParam = indParams->getParameter("ShowMode");
                    modeParam->value()->setString("HIST");
                    
                    const char* err;
                    IIndicatorInstance* instance = indicator->getInstance(barSource, false, indParams, &err);

                    instance->updateAll(&err);
                    WaitForSingleObject(host->getLoaderFinishedEvent(), 60000);

                    PrintResults(instance);
                }
                else
                    std::cout << "Indicator not found" << std::endl;
                delete barSource;
            }
            else
                std::cout << "Invalid timeframe" << std::endl;
        }
        else
            std::cout << "Invalid instrument" << std::endl;

        manager->release();
        delete host;
    }
    else
        std::cout << "login failed" << std::endl;

    return 0;
}

bool Login(IO2GSession *session, SessionStatusListener *statusListener, LoginParams *loginParams)
{
    session->login(loginParams->getLogin(), loginParams->getPassword(),
                   loginParams->getURL(), loginParams->getConnection());

    bool bConnected = false;

    while (true)
    {
        WaitForSingleObject(statusListener->getEvent(), 250);
        if (statusListener->getStatusCode() == IO2GSessionStatus::Connected)
        {
            bConnected = true;
            break;
        }
        if (statusListener->hasError())
        {
            std::cout << "Connection failed: " << statusListener->getError() << std::endl;
            break;
        }
    }
    return bConnected;
}

void Logout(IO2GSession *session, SessionStatusListener *listener)
{
    session->logout();
    while (true)
    {
        WaitForSingleObject(listener->getEvent(), 250);
        if (listener->getStatusCode() == IO2GSessionStatus::Disconnected)
            break;
    }
}

IO2GOfferRow *GetOffer(IO2GSession *session, const char *instrument)
{
    if (!session || !instrument)
        return NULL;

    O2G2Ptr<IO2GLoginRules> loginRules = session->getLoginRules();
    if (loginRules)
    {
        O2G2Ptr<IO2GResponse> response = loginRules->getTableRefreshResponse(Offers);
        if (response)
        {
            O2G2Ptr<IO2GResponseReaderFactory> readerFactory = session->getResponseReaderFactory();
            if (readerFactory)
            {
                O2G2Ptr<IO2GOffersTableResponseReader> reader = readerFactory->createOffersTableReader(response);

                for (int i = 0; i < reader->size(); ++i)
                {
                    O2G2Ptr<IO2GOfferRow> offer = reader->getRow(i);
                    if (offer)
                        if (strcmp(instrument, offer->getInstrument()) == 0)
                            if (strcmp(offer->getSubscriptionStatus(), "V") == 0 ||
                                strcmp(offer->getSubscriptionStatus(), "T") == 0)
                                return offer.Detach();
                }
            }
        }
    }
    return NULL;
}

const char* FormatDate(double dt)
{
    static char buf[128];
    SYSTEMTIME st;

    VariantTimeToSystemTime(dt, &st);
    sprintf_s(buf, 128, "%04d/%02d/%02d %02d:%02d:%02d", st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);

    return buf;
}

void PrintResults(IIndicatorInstance *instance)
{
    IIndicatorOutputStreams *outputs = instance->getStreams();
    IIndicatorBarSource *barSource = (IIndicatorBarSource*)instance->getSource();
    
    std::cout.precision(barSource->getPrecision());
    int iCellWidth = barSource->getPrecision() + 2;

    /** Print table header */
    std::cout << instance->getName() << std::endl;
    std::cout << std::setw(20) << "Date" << "; "
              << std::setw(iCellWidth) << "Open" << "; "
              << std::setw(iCellWidth) << "High" << "; "
              << std::setw(iCellWidth) << "Low" << "; "
              << std::setw(iCellWidth) << "Close" << "; ";
    for(int i = 0; i < outputs->size(); i++)
    {
        IIndicatorOutputStream *stream = outputs->getStream(i);
        std::cout << std::setw(iCellWidth) << stream->getLabel() << "; ";
    }
    std::cout << std::endl;

    int iTableRows = barSource->size();
    for (int i = 0; i < outputs->size(); ++i)
        iTableRows = max(iTableRows, outputs->getStream(i)->size());

    /** print table contents. */
    double open, high, low, close;
    for (int i = 0; i < iTableRows; i++)
    {
        if (i < barSource->size())
        {
            barSource->getOHLC(i, open, high, low, close);
            std::cout << std::setw(20) << FormatDate(barSource->date(i)) << "; "
                      << std::setw(iCellWidth) << std::fixed << open << "; "
                      << std::setw(iCellWidth) << std::fixed << high << "; "
                      << std::setw(iCellWidth) << std::fixed << low << "; "
                      << std::setw(iCellWidth) << std::fixed << close << "; ";
        }
        else
            for (int j = 0; j < 4; ++j)
                std::cout << std::setw(iCellWidth) << "n/a" << "; ";

        for (int j = 0; j < outputs->size(); ++j)
            if (i < outputs->getStream(j)->size())
                std::cout << std::setw(iCellWidth) << std::fixed << outputs->getStream(j)->getTick(i) << "; ";
            else
                std::cout << std::setw(iCellWidth) << "n/a" << "; ";

        std::cout << std::endl;
    }
}

bool CheckObligatoryParams(LoginParams *loginParams, SampleParams *sampleParams)
{
    bool bWasError = false;
    /* Check login params. */
    if(strlen(loginParams->getLogin()) == 0)
    {
        std::cout << LoginParams::Strings::loginNotSpecified << std::endl;
        bWasError = true;
    }
    if(strlen(loginParams->getPassword()) == 0)
    {
        std::cout << LoginParams::Strings::passwordNotSpecified << std::endl;
        bWasError = true;
    }
    if(strlen(loginParams->getURL()) == 0)
    {
        std::cout << LoginParams::Strings::urlNotSpecified << std::endl;
        bWasError = true;
    }
    if(strlen(loginParams->getConnection()) == 0)
    {
        std::cout << LoginParams::Strings::connectionNotSpecified << std::endl;
        bWasError = true;
    }

    /* Check sample params. */
    if(strlen(sampleParams->getIndicatorsPath()) == 0)
    {
        std::cout << SampleParams::Strings::indicatorsPathNotSpecified << std::endl;
        bWasError = true;
    }
    if(strlen(sampleParams->getInstrument()) == 0)
    {
        std::cout << SampleParams::Strings::instrumentNotSpecified << std::endl;
        bWasError = true;
    }
    if(strlen(sampleParams->getTimeframe()) == 0)
    {
        std::cout << SampleParams::Strings::timeframeNotSpecified << std::endl;
        bWasError = true;
    }
    if(sampleParams->getDateFrom() == 0)
    {
        std::cout << SampleParams::Strings::dateFromNotSpecified << std::endl;
        bWasError = true;
    }
    return !bWasError;
}

void PrintHelp()
{
    std::cout << "Usage: \n\
/l | -l | /login | --login\n\
    - Your login. Mandatory argument.\n\
\n\
/p | -p | /password | --password\n\
    - Your password. Mandatory argument.\n\
\n\
/u | -u | /url | --url\n\
    - The connection URL. Mandatory argument.\n\
      The URL must be full, including the path to the host descriptor.\n\
      For example, \"--url http://www.fxcorporate.com/Hosts.jsp\".\n\
\n\
/c | -c | /connection | --connection\n\
    - The connection name. Mandatory argument.\n\
      For example, \"/c Demo\" or \"/connection Real\".\n\
\n\
/subsession | --subsession\n\
    - The database name. Optional argument.\n\
      Required only for users who have a multiple database login.\n\
\n\
/pin | --pin\n\
    - Your pin code. Optional argument.\n\
      Required only for users who have a pin.\n\
\n\
/i | -i | /instrument | --instrument\n\
    - An instrument for which you intend to calculate an indicator.\n\
      For example, \"-i EUR/USD\". Mandatory argument.\n\
\n\
/timeframe | --timeframe\n\
    - A time period that forms a single bar. Mandatory argument.\n\
      Possible values: m1, m5, m15, m30, H1, H2, H4, H6, H8, H12, D1, W1, M1\n\
\n\
/indicators | --indicators\n\
    - Path to the directory with indicators. Mandatory argument.\n\
      For example, \"--indicators C:\\\\Program\\ Files\\\\Candleworks\\\\FXTS2\\\\indicators\\\\standard\"\n\
      or \"/indicators C:\\\\Gehtsoft\\\\IndicoreSDK\\\\indicators\\\\standard\"\n\
      NOTE: do not forget to put escaped backslashes and spaces!\n\
\n\
/csv | --csv\n\
    - Path to the CSV file with price data\n\
      Optional argument. If it is not specified, all data downloads via ForexConnect.\n\
\n\
/datefrom | --datefrom\n\
    - Start downloading bars from this date. Mandatory argument.\n\
      NOTE: format is MM.DD.YYYY hh:mm:ss\n\
      NOTE: specify date in brackets or use escaped space between date and time\n\
      For example: \"--datefrom \"01.01.1970 10:10:38\"\" or \"--datefrom 01.01.1970\\ 10:10:38\"\n\
\n\
/dateto | --dateto\n\
    - Download bars to this date. Optional argument.\n\
      If it is not specified, bars download to this date and bar source is \"live\".\n\
      NOTE: format is MM.DD.YYYY hh:mm:ss\n\
      NOTE: specify date in brackets or use escaped space between date and time\n\
      For example: \"--datefrom \"01.01.1970 10:10:38\"\" or \"--datefrom 01.01.1970\\ 10:10:38\"\n\
\n\
/bid | --bid\n\
    - Boolean flag. Pass it to the application to work with bid bars, or do not pass to work with ask bars.\n\
      " << std::endl;
}

} // end of local unnamed namespace

// Entry point of application
int main(int argc, char* argv[])
{
    return ForexConnectHost(argc, argv);
}