#include "stdafx.h"
#include "SessionStatusListener.h"

/** Constructor. */
SessionStatusListener::SessionStatusListener(IO2GSession *session, bool printSubsessions, const char *subSessionID, const char *pin)
{
    if (subSessionID != 0)
        mSubSessionID = subSessionID;
    else
        mSubSessionID = "";
    if (pin != 0)
        mPin = pin;
    else
        mPin = "";
    mSession = session;
    mSession->addRef();
    mHasError = false;
    mError = "";
    mPrintSubsessions = printSubsessions;
    mRefCount = 1;
    mStatusCode = IO2GSessionStatus::Disconnected;
    mEvent = CreateEvent(0, FALSE, FALSE, 0);
}

/** Destructor. */
SessionStatusListener::~SessionStatusListener()
{
    mSession->release();
    mSubSessionID.clear();
    mPin.clear();
    mError.clear();
    CloseHandle(mEvent);
}

/** Increase reference counter. */
long SessionStatusListener::addRef()
{
    return InterlockedIncrement(&mRefCount);
}

/** Decrease reference counter. */
long SessionStatusListener::release()
{
    long rc = InterlockedDecrement(&mRefCount);
    if (rc == 0)
        delete this;
    return rc;
}

/** Callback called when login has been failed. */
void  SessionStatusListener::onLoginFailed(const char *error)
{
    mHasError = true;
    mError = error;
    SetEvent(mEvent);
}

/** Callback called when session status has been changed. */
void SessionStatusListener::onSessionStatusChanged(IO2GSessionStatus::O2GSessionStatus status)
{
    switch (status)
    {
    case    IO2GSessionStatus::Disconnected:
            std::cout << "status::disconnected" << std::endl;
            break;
    case    IO2GSessionStatus::Connecting:
            std::cout << "status::connecting" << std::endl;
            break;
    case    IO2GSessionStatus::TradingSessionRequested:
            std::cout << "status::trading session requested" << std::endl;
            break;
    case    IO2GSessionStatus::Connected:
            std::cout << "status::connected" << std::endl;
            break;
    case    IO2GSessionStatus::Reconnecting:
            std::cout << "status::reconnecting" << std::endl;
            break;
    case    IO2GSessionStatus::Disconnecting:
            std::cout << "status::disconnecting" << std::endl;
            break;
    case    IO2GSessionStatus::SessionLost:
            std::cout << "status::session lost" << std::endl;
            break;
    }

    if (status == IO2GSessionStatus::TradingSessionRequested)
    {
        O2G2Ptr<IO2GSessionDescriptorCollection> descriptors = mSession->getTradingSessionDescriptors();

        bool found = false;
        if (descriptors)
        {
            if (mPrintSubsessions)
                std::cout << "descriptors available:" << std::endl;
            for (int i = 0; i < descriptors->size(); i++)
            {
                O2G2Ptr<IO2GSessionDescriptor> descriptor = descriptors->get(i);

                if (mPrintSubsessions)
                    std::cout << "  id:='" << descriptor->getID()
                              << "' name='" << descriptor->getName()
                              << "' description='" << descriptor->getDescription()
                              << "' " << (descriptor->requiresPin() ? "requires pin" : "") << std::endl;

                if (mSubSessionID == descriptor->getID())
                {
                    found = true;
                    break;
                }
            }
        }

        if (!found)
        {
            onLoginFailed("The specified sub session identifier is not found");
        }
        else
        {
            mSession->setTradingSession(mSubSessionID.c_str(), mPin.c_str());
        }
    }

    mStatusCode = status;
    SetEvent(mEvent);
}

/** Get the session status. */
IO2GSessionStatus::O2GSessionStatus SessionStatusListener::getStatusCode() const
{
    return mStatusCode;
}

/** Check whether error happened. */
bool SessionStatusListener::hasError() const
{
    return mHasError;
}

/** Check whether session is connected */
bool SessionStatusListener::isConnected() const
{
    return mStatusCode == IO2GSessionStatus::Connected;
}

/** Check whether session is disconnected */
bool SessionStatusListener::isDisconnected() const
{
    return mStatusCode == IO2GSessionStatus::Disconnected;
}

/** Reset error information (use before login). */
void SessionStatusListener::resetError()
{
    mHasError = false;
    mError = "";
}

/** Get error. */
const char *SessionStatusListener::getError() const
{
    return mError.c_str();
}

HANDLE SessionStatusListener::getEvent() const
{
    return mEvent;
}

